/* 
 * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 * Copyright (C) 2005/2006, Anthony Minessale II <anthmct@yahoo.com>
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 *
 * The Initial Developer of the Original Code is
 * Anthony Minessale II <anthmct@yahoo.com>
 * Portions created by the Initial Developer are Copyright (C)
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Anthony Minessale II <anthmct@yahoo.com>
 * Neal Horman <neal at wanlink dot com>
 *
 *
 * mod_powerdial.c -- Framework Demo Module
 *
 */
#include <switch.h>

SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_powerdial_shutdown);
SWITCH_MODULE_RUNTIME_FUNCTION(mod_powerdial_runtime);
SWITCH_MODULE_LOAD_FUNCTION(mod_powerdial_load);
SWITCH_MODULE_DEFINITION(mod_powerdial, mod_powerdial_load, mod_powerdial_shutdown, mod_powerdial_runtime);

#define PD_QUEUE_LEN 200000
#define DEFAULT_THREAD_COUNT 10

static struct {
	switch_queue_t *queue;
	int threads_wanted;
	int threads_running;
	switch_mutex_t *mutex;
	int done;
	int id_pool;
	int main_running;
	switch_memory_pool_t *pool;
} globals;




static void *SWITCH_THREAD_FUNC event_thread(switch_thread_t *thread, void *obj)
{
	void *pop;
	int id = 0;

	switch_mutex_lock(globals.mutex);
	globals.threads_running++;
	id = ++globals.id_pool;
	switch_mutex_unlock(globals.mutex);
	
	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "Thread %d started\n", id);
		
	for(;;) {
		char *id, *ds;
		switch_call_cause_t cause = SWITCH_CAUSE_NONE;

		if (switch_queue_pop(globals.queue, &pop) != SWITCH_STATUS_SUCCESS) {
			break;
		}

		if (!pop || globals.done) {
			break;
		}

		id = (char *) pop;
		if ((ds = strchr(id, ':'))) {
			*ds++ = '\0';
		}
		
		if (ds) {
			switch_stream_handle_t stream = { 0 };
			char *cmd = ds;
			char *arg = NULL;
			char *reply;
			const char *cause_str;
			

			if ((arg = strchr(cmd, ' '))) {
				*arg++ = '\0';
			}

			SWITCH_STANDARD_STREAM(stream);
			switch_api_execute(cmd, arg, NULL, &stream);
			
			reply = (char *) stream.data;

			if (!strncasecmp(reply, "-err ", 5)) {
				cause_str = reply + 5;
				if (!strcasecmp(cause_str, "cannot")) {
					if ((cause_str = switch_stristr("cause: ", reply))) {
						cause_str += 7;
					}
				}

				if (!(cause = switch_channel_str2cause(cause_str))) {
					cause = SWITCH_CAUSE_USER_BUSY;
				}
			}
			
			switch_safe_free(stream.data);
		}

		// do sql stmt etc here 
		if (cause) { //didnt work
		} else { //worked
		}
		
		free(pop);
	}
	
	switch_mutex_lock(globals.mutex);
	globals.threads_running--;
	switch_mutex_unlock(globals.mutex);
	
	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "Thread %d ended\n", id);
	
	return NULL;
}


static void launch_threads(void)
{
	switch_thread_t *thread;
	switch_threadattr_t *thd_attr;
	int x = 0;
	switch_threadattr_create(&thd_attr, globals.pool);
	switch_threadattr_detach_set(thd_attr, 1);

	switch_threadattr_stacksize_set(thd_attr, SWITCH_THREAD_STACKSIZE);
    //switch_threadattr_priority_increase(thd_attr);

	for(x = 0; x < globals.threads_wanted; x++) {
		switch_thread_create(&thread, thd_attr, event_thread, NULL, globals.pool);	
	}
}


SWITCH_MODULE_LOAD_FUNCTION(mod_powerdial_load)
{

	memset(&globals, 0, sizeof(globals));

	*module_interface = switch_loadable_module_create_module_interface(pool, modname);
	switch_queue_create(&globals.queue, PD_QUEUE_LEN, pool);
	switch_mutex_init(&globals.mutex, SWITCH_MUTEX_NESTED, pool);

	globals.pool = pool;
	/* indicate that the module should continue to be loaded */
	return SWITCH_STATUS_SUCCESS;
}


SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_powerdial_shutdown)
{
	void *pop;
	int x = 0;

	switch_mutex_lock(globals.mutex);
	globals.done = 1;
	globals.main_running = -1;
	switch_mutex_unlock(globals.mutex);

	for(x = 0; x < globals.threads_running; x++) {
		switch_queue_push(globals.queue, NULL);
	}
	
	while(globals.threads_running && globals.main_running) {
		switch_yield(500000);
	}

	while (switch_queue_trypop(globals.queue, &pop) == SWITCH_STATUS_SUCCESS) {
		if (pop) {
			free(pop);
		}
	}

	return SWITCH_STATUS_SUCCESS;
}


SWITCH_MODULE_RUNTIME_FUNCTION(mod_powerdial_runtime)
{
	char *dynamic_string = NULL;
	
	// set some shit up

	// start the threads

	globals.main_running = 1;
	globals.threads_wanted = DEFAULT_THREAD_COUNT;
	launch_threads();
	
	while(globals.main_running == 1) {
		// do your shit here jamming shit into the queue
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "w00t\n");
		dynamic_string = switch_mprintf("xyz:originate sofia/default/foo@bar.com 1234");
		switch_queue_push(globals.queue, dynamic_string);
		
		switch_yield(5000000);
	}

	globals.main_running = 0;
	
	return SWITCH_STATUS_TERM;
}


/* For Emacs:
 * Local Variables:
 * mode:c
 * indent-tabs-mode:t
 * tab-width:4
 * c-basic-offset:4
 * End:
 * For VIM:
 * vim:set softtabstop=4 shiftwidth=4 tabstop=4 expandtab:
 */
